/*
 * Copyright 2002-2006 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.springframework.config.java.parsing;

import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.RootBeanDefinition;
import org.springframework.config.java.annotation.Configuration;

/**
 * <a href="http://asm.objectweb.org">ASM</a> based annotation bean definition
 * reader. This implementation will read the bytecode directly to find more
 * information about each class read. ASM will be used as much as possible in
 * order to avoid unnecessary class loading.
 * 
 * @author Costin Leau
 * @author Rod Johnson
 */
public class ConfigurationClassScanningBeanDefinitionReader extends AbstractAsmClassScanningBeanDefinitionReader {

	public ConfigurationClassScanningBeanDefinitionReader(BeanDefinitionRegistry beanFactory) {
		super(beanFactory);
		addTypeFilter(new AnnotationTypeFilter(Configuration.class));
	}

	protected void processComponentOrFactoryClass(Class clazz) {
		if (log.isDebugEnabled())
			log.debug("processing class " + clazz);
		BeanDefinition definition = new RootBeanDefinition(clazz);
		getBeanFactory().registerBeanDefinition(clazz.getName(), definition);
	}
}
