/*******************************************************************************
 * Copyright (c) 2005, 2007 Spring IDE Developers
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Spring IDE Developers - initial API and implementation
 *******************************************************************************/
package org.springframework.ide.eclipse.webflow.core.internal.model;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.wst.xml.core.internal.provisional.document.IDOMNode;
import org.springframework.ide.eclipse.core.model.IModelElement;
import org.springframework.ide.eclipse.core.model.IModelElementVisitor;
import org.springframework.ide.eclipse.webflow.core.model.IInputAttribute;
import org.springframework.ide.eclipse.webflow.core.model.IWebflowModelElement;

/**
 * @author Christian Dupuis
 * @since 2.0
 */
@SuppressWarnings("restriction")
public class InputAttribute extends AbstractModelElement implements IInputAttribute {

	public String getName() {
		return getAttribute("name");
	}

	public boolean getRequired() {
		return Boolean.valueOf(getAttribute("required"));
	}

	public String getScope() {
		return getAttribute("scope");
	}

	public void setName(String name) {
		setAttribute("name", name);
	}

	public void setRequired(boolean required) {
		if (required) {
			setAttribute("required", "true");
		}
		else {
			setAttribute("required", "false");
		}
	}

	public void setScope(String scope) {
		setAttribute("scope", scope);
	}

	public void createNew(IWebflowModelElement parent) {
		IDOMNode node = null;
		if (WebflowModelXmlUtils.isVersion1Flow(this)) {
			node = (IDOMNode) parent.getNode().getOwnerDocument().createElement("input-attribute");
		}
		else {
			node = (IDOMNode) parent.getNode().getOwnerDocument().createElement("input");
		}
		init(node, parent);
	}

	public void accept(IModelElementVisitor visitor, IProgressMonitor monitor) {
		visitor.visit(this, monitor);
	}

	public IModelElement[] getElementChildren() {
		List<IModelElement> children = new ArrayList<IModelElement>();
		return children.toArray(new IModelElement[children.size()]);
	}

	public String getType() {
		return getAttribute("type");
	}

	public String getValue() {
		return getAttribute("value");
	}

	public void setType(String type) {
		setAttribute("type", type);
	}

	public void setValue(String value) {
		setAttribute("value", value);
	}
}
